/*
   This program creates the homotopy between a circle and a square
   but it shows a bit more detail than the previous version.
*/
#include <windows.h>
#include <math.h>
#include "glut.h"
#include "glui.h"

#define N 25
#define PI 3.14159265
double circleX[N+1], circleY[N+1], squareX[N+1], squareY[N+1];

/* GLUT callback function prototypes */
void display(void);
void reshape(int, int);
void slider(float);

int windowHeight;         // this is set by the glut reshape() callback function
int windowWidth;          // this is set by the glut reshape() callback function

#define sliderInit 0.0
float sliderValue = sliderInit;  // this is set by the glui slider() callback function

int main(int argc, char** argv)
{
   int i;
   int window;
   glutInit(&argc, argv);
   glutInitDisplayMode(GLUT_DOUBLE | GLUT_RGB);  // glui needs double buffering!
   // specify the screen window
   glutInitWindowSize(500,500);       // width, height (in pixels)
   glutInitWindowPosition(100,150);   // left, top (in pixels)
   window = glutCreateWindow("Homotopy v3");
   glutDisplayFunc(display);
   glutReshapeFunc(reshape);
   /* set up viewing */
   glMatrixMode(GL_PROJECTION);
   glLoadIdentity();
   gluOrtho2D(-2.0, 2.0, -2.0, 2.0); /* viewing volume */
   glMatrixMode(GL_MODELVIEW);

   // parent window, x, y, width, height, initial value, callback function
   gluiHorizontalSlider(window, 10, 0, -10, 20, sliderInit, slider);

   glClearColor(1.0, 1.0, 1.0, 0.0);
   glLineWidth(1.0);

   for (i = 0; i <= N; i++)  // compute values up to the angle Pi/4 (45 degrees)
   {
      circleX[i] = cos((i/(double)N)*PI/4.0);
      circleY[i] = sin((i/(double)N)*PI/4.0);
      squareX[i] = 1.0;
      squareY[i] = tan((i/(double)N)*PI/4.0);
   }

   glutMainLoop( );

   return 0;
}//main()


void display(void)
{
   int i;
   double X, Y;
   float t = sliderValue;

   // specify the viewport
   glViewport( 0,  0, windowWidth, windowHeight); // left, bottom, width, height (in pixels)

   glClear( GL_COLOR_BUFFER_BIT );

   glColor3f(0.0f, 1.0f, 0.0f);

   // draw the radial "homotopy lines"
   glBegin(GL_LINES);
      for(i = 0; i <= N; i++)   // 1st octant
      {
         glVertex2f(circleX[i], circleY[i]);
         glVertex2f(squareX[i], squareY[i]);
      }
      for(i = N; i > 0; i--)  // 2nd octant
      {
         glVertex2f(circleY[i], circleX[i]);
         glVertex2f(squareY[i], squareX[i]);
      }
      for(i = 0; i <= N; i++)   // 3rd octant
      {
         glVertex2f(-circleY[i], circleX[i]);
         glVertex2f(-squareY[i], squareX[i]);
      }
      for(i = N-1; i > 0; i--)  // 4th octant
      {
         glVertex2f(-circleX[i], circleY[i]);
         glVertex2f(-squareX[i], squareY[i]);
      }
      for(i = 0; i <= N; i++)   // 5th octant
      {
         glVertex2f(-circleX[i], -circleY[i]);
         glVertex2f(-squareX[i], -squareY[i]);
      }
      for(i = N-1; i > 0; i--)  // 6th octant
      {
         glVertex2f(-circleY[i], -circleX[i]);
         glVertex2f(-squareY[i], -squareX[i]);
      }
      for(i = 0; i <= N; i++)   // 7th octant
      {
         glVertex2f(circleY[i], -circleX[i]);
         glVertex2f(squareY[i], -squareX[i]);
      }
      for(i = N-1; i > 0; i--)  // 8th octant
      {
         glVertex2f(circleX[i], -circleY[i]);
         glVertex2f(squareX[i], -squareY[i]);
      }
   glEnd();

   glColor3f(1.0f, 0.0f, 0.0f);

   // draw the circle (one octant at a time)
   glBegin(GL_LINE_STRIP);
      for(i = 0; i <= N; i++)   // 1st octant
      {
         glVertex2f(circleX[i], circleY[i]);
      }
      for(i = N-1; i > 0; i--)  // 2nd octant
      {
         glVertex2f(circleY[i], circleX[i]);
      }
      for(i = 0; i <= N; i++)   // 3rd octant
      {
         glVertex2f(-circleY[i], circleX[i]);
      }
      for(i = N-1; i > 0; i--)  // 4th octant
      {
         glVertex2f(-circleX[i], circleY[i]);
      }
      for(i = 0; i <= N; i++)   // 5th octant
      {
         glVertex2f(-circleX[i], -circleY[i]);
      }
      for(i = N-1; i > 0; i--)  // 6th octant
      {
         glVertex2f(-circleY[i], -circleX[i]);
      }
      for(i = 0; i <= N; i++)   // 7th octant
      {
         glVertex2f(circleY[i], -circleX[i]);
      }
      for(i = N-1; i > 0; i--)  // 8th octant
      {
         glVertex2f(circleX[i], -circleY[i]);
      }
   glEnd();

   // draw the square (one octant at a time)
   glBegin(GL_LINE_STRIP);
      for(i = 0; i <= N; i++)   // 1st octant
      {
         glVertex2f(squareX[i], squareY[i]);
      }
      for(i = N-1; i > 0; i--)  // 2nd octant
      {
         glVertex2f(squareY[i], squareX[i]);
      }
      for(i = 0; i <= N; i++)   // 3rd octant
      {
         glVertex2f(-squareY[i], squareX[i]);
      }
      for(i = N-1; i > 0; i--)  // 4th octant
      {
         glVertex2f(-squareX[i], squareY[i]);
      }
      for(i = 0; i <= N; i++)   // 5th octant
      {
         glVertex2f(-squareX[i], -squareY[i]);
      }
      for(i = N-1; i > 0; i--)  // 6th octant
      {
         glVertex2f(-squareY[i], -squareX[i]);
      }
      for(i = 0; i <= N; i++)   // 7th octant
      {
         glVertex2f(squareY[i], -squareX[i]);
      }
      for(i = N-1; i > 0; i--)  // 8th octant
      {
         glVertex2f(squareX[i], -squareY[i]);
      }
   glEnd();

   glColor3f(0.0f, 0.0f, 1.0f);

   // draw the curve interpolated between the circle and the square
   // (one octant at a time)
   glBegin(GL_LINE_STRIP);
      for(i = 0; i <= N; i++)    // 1st octant
      {
         X = (1-t)*circleX[i]+t*squareX[i];  // lerp the x-coordinate
         Y = (1-t)*circleY[i]+t*squareY[i];  // lerp the y-coordinate
         glVertex2f(X, Y);
      }
      for(i = N-1; i > 0; i--)  // 2nd octant
      {
         X = (1-t)*circleX[i]+t*squareX[i];
         Y = (1-t)*circleY[i]+t*squareY[i];
         glVertex2f(Y, X);
      }
      for(i = 0; i <= N; i++)    // 3rd octant
      {
         X = (1-t)*circleX[i]+t*squareX[i];
         Y = (1-t)*circleY[i]+t*squareY[i];
         glVertex2f(-Y, X);
      }
      for(i = N-1; i > 0; i--)  // 4th octant
      {
         X = (1-t)*circleX[i]+t*squareX[i];
         Y = (1-t)*circleY[i]+t*squareY[i];
         glVertex2f(-X, Y);
      }
      for(i = 0; i <= N; i++)    // 5th octant
      {
         X = (1-t)*circleX[i]+t*squareX[i];
         Y = (1-t)*circleY[i]+t*squareY[i];
         glVertex2f(-X, -Y);
      }
      for(i = N-1; i > 0; i--)  // 6th octant
      {
         X = (1-t)*circleX[i]+t*squareX[i];
         Y = (1-t)*circleY[i]+t*squareY[i];
         glVertex2f(-Y, -X);
      }
      for(i = 0; i <= N; i++)    // 7th octant
      {
         X = (1-t)*circleX[i]+t*squareX[i];
         Y = (1-t)*circleY[i]+t*squareY[i];
         glVertex2f(Y, -X);
      }
      for(i = N-1; i > 0; i--)  // 8th octant
      {
         X = (1-t)*circleX[i]+t*squareX[i];
         Y = (1-t)*circleY[i]+t*squareY[i];
         glVertex2f(X, -Y);
      }
   glEnd();

   glutSwapBuffers();  // glui needs double buffering!
}


/* The reshape() callback function */
void reshape(int width, int height)
{
   windowWidth  = width;
   windowHeight = height;

   printf("We got a \"reshape(%d, %d)\" event\n", width, height);

   gluiReshape(width, height);  // notice the difference here
}//reshape()


/* glui slider callback function */
void slider(float value)
{
   sliderValue = value;

   printf("We got a \"slider(%f)\" event\n", value);

   glutPostRedisplay();  // notice the difference here
}//slider()